/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <unistd.h>

//#include <glib.h>

#include "glib2_compat.h"

#include "cmdline.h"
#include "configuration.h"
#include "log.h"


/*! The command line arguments repository. */
cmdline_t *cmdline_args = NULL;


/**
 * Initializes everything related to command line parsing.
 */
void cmdline_init() 
{
  log_debug("[cmdline init] init");

  cmdline_args = cmdline_new();

  log_debug("[cmdline init] exit");
}


/**
 * Destroy the data structures and free used memory.
 */
void cmdline_finalize()
{
  log_debug("[cmdline finalize] init");

  cmdline_destroy(cmdline_args);
  cmdline_args = NULL;

  log_debug("[cmdline finalize] exit");
}


/**
 * Informs if this modules has been initialized.
 * \return A boolean that indicates if this modules has been initialized.
 */
gboolean cmdline_enabled()
{
  return cmdline_args != NULL;
}


/**
 *
 * \return
 */
cmdline_t* cmdline_new()
{
  cmdline_t *cmdline = g_new0(cmdline_t, 1);
  cmdline->cmd = NULL;
  cmdline->config_filename = NULL;
  cmdline->debug = FALSE;
  cmdline->daemon = FALSE;

  return cmdline;
}


/**
 * 
 * \param cmdline
 */
void cmdline_destroy(cmdline_t* cmdline)
{
  g_free(cmdline);
}


/**
 * Prints how this program should be called.
 * \param cmd The command passed to call this program.
 */
void cmdline_usage(const gchar* const cmd) {
  G_FPRINTF(stderr, 
	  "Usage:\t%s [options]\n"
	  "\t-h\t\t: Print this help message.\n"
	  "\t-c cfg-file\t: Read configuration from cfg-file,\n"
	  "\t\t\t  default is '%s'.\n"
	  //	  "\t-n nodename\t: Use nodename instead of real hostname\n"
	  //	  "\t\t\t  to identify this instance.\n"
	  "\t-g\t\t: Run in the debug mode (foreground).\n"
	  //"\t\t\t- foreground and\n"
	  //"\t\t\t  virtual node support.\n"
	  "\t-d\t\t: Run in daemon mode.\n",
	  //	  "\t-l log-level\t: Used log-level for logging to stderr,\n"
	  //	  "\t\t\t  default is 0, i.e., no logging.\n",
	  g_basename(cmd), CONFIG_FILENAME_DEFAULT);
}


/**
 * Parse the command line arguments.
 * \param argc The number of argumens in the command line.
 * \param argv The values of the arguments in the command line.
 * \return An indication of failure or success of parsing the arguments.
 */
gboolean cmdline_parse(const guint argc, gchar** const argv) { 
  gchar c;

  cmdline_args->cmd = argv[0];

  while (TRUE) {
    c = getopt(argc, argv, CMDLINE_OPTS);
    if (c == -1) {
      break;
    }
    
    switch (c) {
    case 'h':
      return FALSE;
      break;

    case 'c':
      cmdline_args->config_filename = optarg;
      break;
      
    case 'g':
      cmdline_args->debug = TRUE;
      break;

    case 'd':
      cmdline_args->daemon = TRUE;
      break;

    default:
      return FALSE;
    }
  }
  
  return TRUE;
}
