/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _CONFIGURATION_H
#define _CONFIGURATION_H

#include <glib.h>

#include "log.h"

/*! Default configuration file name. */
#define CONFIG_FILENAME_DEFAULT "/usr/local/etc/gmsd/gmsd.conf"
/*! Default node list file name. */
#define NODE_LIST_FILE_DEFAULT  "/usr/local/etc/gmsd/node_list"

/*! configuration parameter of the communication port. */
#define CONFIG_OPT_PORT               "comm.port"
/*! configuration parameter of the multicast address. */
#define CONFIG_OPT_MCAST_ADDRESS      "comm.mcast.address"
/*! configuration parameter of the multicast port number. */
#define CONFIG_OPT_MCAST_PORT         "comm.mcast.port"
/*! configuration parameter of the multicast time to live. */
#define CONFIG_OPT_MCAST_TTL          "comm.mcast.ttl"
/*! configuration parameter of the multicast loopback option. */
#define CONFIG_OPT_MCAST_LOOP         "comm.mcast.loop"
/*! configuration parameter of the debug mode. */
#define CONFIG_OPT_DEBUG              "debug"
/*! configuration parameter of the node list file name. */
#define CONFIG_OPT_NODE_LIST_FILE     "node.list.file"
/*! configuration parameter of the neighbor's timer timeout. */
#define CONFIG_OPT_TIMEOUT_DELAY      "timeout.delay"
/*! configuration parameter of the delay's timer timeout. */
#define CONFIG_OPT_TIMEOUT_NEIGHBOR   "timeout.neighbor"
/*! configuration parameter of the probe's timer timeout. */
#define CONFIG_OPT_TIMEOUT_PROBE      "timeout.probe"
/*! configuration parameter of the log file. */
#define CONFIG_OPT_LOG_FILENAME       "log.filename"
/*! configuration parameter for the log level.  */
#define CONFIG_OPT_LOG_LEVEL          "log.level"
/*! configuration parameter for the GID persistence. */
#define CONFIG_OPT_GROUP_ID_FILENAME  "gid.filename"
/*! configuration parameter for the membership persistence */ 
#define CONFIG_OPT_IPC_FILENAME       "ipc.filename"
/*! configuration parameter for the user to run the daemon. */ 
#define CONFIG_OPT_RUN_AS_USER        "run_as.user"
/*! configuration parameter for the group to run the daemon.  */ 
#define CONFIG_OPT_RUN_AS_GROUP       "run_as.group"

/*! */
#define CONFIG_LOG_LEVEL_ERROR     LOG_LEVEL_ERROR
/*! */
#define CONFIG_LOG_LEVEL_CRITICAL  LOG_LEVEL_CRITICAL
/*! */
#define CONFIG_LOG_LEVEL_WARNING   LOG_LEVEL_WARNING
/*! */
#define CONFIG_LOG_LEVEL_NOTICE    LOG_LEVEL_NOTICE
/*! */
#define CONFIG_LOG_LEVEL_INFO      LOG_LEVEL_INFO
/*! */
#define CONFIG_LOG_LEVEL_DEBUG     LOG_LEVEL_DEBUG
/*! */
#define CONFIG_LOG_LEVEL_OPTIONS "{" CONFIG_LOG_LEVEL_ERROR "," CONFIG_LOG_LEVEL_CRITICAL "," \
                                     CONFIG_LOG_LEVEL_WARNING "," CONFIG_LOG_LEVEL_NOTICE "," \
                                     CONFIG_LOG_LEVEL_INFO "," CONFIG_CONFIG_LOG_LEVEL_DEBUG "}"

void config_init(gchar* const);
void config_finalize(void);
gboolean config_enabled(void);

void config_read(void);

#endif /* configuration.h */
