/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _GID_H
#define _GID_H

//#include <glib.h>
//#include <libxml/tree.h>

#include "node.h"

/*! */
typedef guint group_id_seq_t;

/*! */
#define GROUP_ID_SEQ_FORMAT "%" G_GUINT32_FORMAT


/*! A group view, used to uniquely identify a group. */
typedef struct {
  group_id_seq_t seq;   /*!< the sequence number. */
  node_id_t node_id;    /*!< the node identifier. */
} group_id_t;


/*! The default file to persist the group id information. */
#define GROUP_ID_FILENAME_DEFAULT  "/tmp/gmsd.gid" 

/*! The first group identifier sequence number to use. */
#define GROUP_ID_INITIAL_ID 0



group_id_t* group_id_new(const group_id_seq_t, const node_id_t);
void group_id_destroy(group_id_t*);
group_id_t* group_id_copy(const group_id_t* const);

void group_id_persist_init(group_id_t*, const gchar* const);
void group_id_persist_destroy(void);
gboolean group_id_persist_enabled(void);

gint group_id_compare(const group_id_t* const, const group_id_t* const);
void group_id_set(group_id_t* const, const group_id_seq_t, const node_id_t);
void group_id_store(group_id_t* const);
void group_id_load(group_id_t* const);

xmlDocPtr group_id_to_xml(const group_id_t* const);
group_id_t* group_id_from_xml(const xmlDocPtr);


#endif /* gid.h */

