/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <unistd.h>

//#include <glib.h>
//#include <libxml/tree.h>

#include "config.h"
#include "ipc.h"
#include "log.h"
//#include "membership.h"
#include "msg.h"

/*! The filename where new memberships should be saved. */
static const gchar* ipc_filename = NULL;


/*! Internal variable used to identify if this module has already been initialized */
static gboolean enabled = FALSE;


/**
 * Initializes the IPC routines.
 * \param filename Where the membership should be saved when updated.
 */
void ipc_init(const gchar* const filename)
{
  log_debug("[ipc init] init");

  ipc_filename = filename;
  enabled = TRUE;

  log_debug("[ipc init] exit");
}

/**
 *
 */
void ipc_finalize()
{
  gint rv;

  log_debug("[ipc finalize] init");

  rv = unlink(ipc_filename);
  g_assert(rv == 0);
  ipc_filename = NULL;

  enabled = FALSE;

  log_debug("[ipc finalize] exit");
}


/**
 * Informs if this modules has been initialized.
 * \return A boolean that indicates if this modules has been initialized.
 */
gboolean ipc_enabled()
{
  return enabled;
}


/**
 * Called when this node has joined a new membership.
 * \param membership The new membership just joined.
 */
void ipc_membership_changed(const membership_t* const membership)
{
  log_debug("[ipc membership changed] init");

#if USE_MSG_ID
 {
   msg_t* cm = msg_get_current_message();
   gchar *msg_id = NULL;
   if (cm != NULL) {
     msg_id = cm->id;
   }
   
   if (log_info_enabled()) {
     gchar* aux = group_to_string(membership->group);
     log_info("joined group (" GROUP_ID_SEQ_FORMAT "," NODE_ID_FORMAT "): {%s} (message " MSG_ID_TYPE_FORMAT ")", 
	      membership->gid->seq, membership->gid->node_id, aux, msg_id);
     g_free(aux);
   }
 }
#else
  if (log_info_enabled()) {
    gchar* aux = group_to_string(membership->group);
    log_info("joined group (" GROUP_ID_SEQ_FORMAT "," NODE_ID_FORMAT "): {%s}", 
	     membership->gid->seq, membership->gid->node_id, aux);
    g_free(aux);
  }
#endif

  /**
   * This routine should alert all local clients about the membership change.
   * For now, it only writes the new membership into a file.
   */
  membership_save(membership, ipc_filename);

  //membership_print(membership);

  log_debug("[ipc membership changed] exit");
}



