/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _LOG_H
#define _LOG_H

#include <glib.h>

/*! The log tag to be used. */
#define LOG_TAG "gmsd"

/*! A flag indicating to log everything. */
#define  LOG_ALL_LEVEL G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION

/*! The length of the date to be printed on log entries. */
#define LOG_DATE_LENGTH 25

/*! The log structure. */
typedef struct {
  const gchar* domain;           /*!< The log domain. */
  GLogLevelFlags level;          /*!< The current log level. */
  guint handler;                 /*!< The log handler. */
  GIOChannel* iochannel;         /*!< The log IO channel. */
  //  node_id_t node_id;             /*!< The local node identifier. */
  gchar* nodename;               /*!< The local nodename (in case it needs to be logged). */
} log_t; 


void log_init(const gint, const gchar* const, const gchar* const);
void log_finalize(void);
gboolean log_enabled(void);

log_t* log_new(const gchar* const, const gchar* const, GLogLevelFlags, const gchar* const, GLogFunc);
void log_destroy(log_t*);

/*! */
#define LOG_FILENAME_DEFAULT "/tmp/gmsd.log"


/*! A string that represents the 'error' log level.*/
#define LOG_LEVEL_ERROR "error"
/*! A string that represents the 'critical' log level.*/
#define LOG_LEVEL_CRITICAL "critical"
/*! A string that represents the 'warning' log level.*/
#define LOG_LEVEL_WARNING "warning"
/*! A string that represents the 'notice' log level.*/
#define LOG_LEVEL_NOTICE "notice"
/*! A string that represents the 'info' log level.*/
#define LOG_LEVEL_INFO "info"
/*! A string that represents the 'debug' log level.*/
#define LOG_LEVEL_DEBUG "debug"


/*! */
#define LOG_LEVEL_ERROR_VALUE    2
/*! */
#define LOG_LEVEL_CRITICAL_VALUE 3
/*! */
#define LOG_LEVEL_WARNING_VALUE  4
/*! */
#define LOG_LEVEL_NOTICE_VALUE   5
/*! */
#define LOG_LEVEL_INFO_VALUE     6
/*! */
#define LOG_LEVEL_DEBUG_VALUE    7


void log_error(const char* const, ...);
void log_critical(const char* const, ...);
void log_warning(const char* const, ...);
void log_notice(const char* const, ...);
void log_info(const char* const, ...);
void log_debug(const char* const, ...);

gboolean log_info_enabled(void);
gboolean log_notice_enabled(void);
gboolean log_warning_enabled(void);
gboolean log_critical_enabled(void);
gboolean log_error_enabled(void);
gboolean log_debug_enabled(void);


#endif /* log.h */
