/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _MEMBERSHIP_H
#define _MEMBERSHIP_H

//#include <glib.h>
//#include <libxml/tree.h>

#include "group.h"
#include "group_id.h"
//#include "node.h"

/*! The timestamp printf format to use. */
#define MEMBERSHIP_TIMESTAMP_FORMAT "%ld"

/*! The definition of a membership. */
typedef struct {  
  group_id_t *gid;        /*!< The group identifier. */
  group_t *group;         /*!< The group itself. */
  GHashTable *timestamps; /*!< The timestamps with this membership. */
} membership_t;


void membership_init(const gchar* const, const guint);
void membership_finalize(void);
gboolean membership_enabled(void);

membership_t* membership_new(group_id_t* const);
void membership_destroy(membership_t*);
void membership_destroy_full(membership_t*);
void membership_print(const membership_t* const);
void membership_save(const membership_t* const, const gchar*);


node_t* membership_get_mynode(void);
node_t* membership_get_node_by_id(const node_id_t);

gboolean membership_majority(const membership_t* const);
node_t * membership_leader(const membership_t* const);
node_t * membership_left_neighbor(const membership_t* const, const node_t*);
node_t * membership_right_neighbor(const membership_t* const, const node_t*);
gint     membership_rank(const membership_t* const, const node_t* const);
gboolean membership_is_leader(const membership_t* const, const node_t* const);

xmlDocPtr membership_to_xml(const membership_t* const);
membership_t* membership_from_xml(const xmlDocPtr);


#endif /* membership.h */
