/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _MSG_H
#define _MSG_H

/* compilation definitions */
#include "config.h"

//#include <glib.h>
//#include <libxml/tree.h>

//#include "node.h"
#include "group.h"

#if USE_MSG_ID

/*! The printf format for handling \ref msg_id_t */
#define MSG_ID_TYPE_FORMAT "%s"

/*! The message identifier. */
typedef gchar* msg_id_t;

#endif


/*! The message type. */
typedef guint msg_type_t;

/*! The printf format for handling \ref msg_type_t */
#define MSG_TYPE_FORMAT "%" G_GUINT32_FORMAT


/*! The recipient type. */
typedef enum {
  MSG_RCPT_NODE = 1,
  MSG_RCPT_GROUP = 2
} msg_rcpt_type_t;

/*! The printf format for handling \ref msg_rcpt_type_t */
#define MSG_RCPT_TYPE_FORMAT "%d" 


/*! The message recipient. */
typedef struct {
  msg_rcpt_type_t type;  /*!< The type of the recipient. */
  union {
    node_t  *node;       /*!< The node to send the message. */
    group_t *group;      /*!< The group to send the message. */
  } to;
} msg_rcpt_t;


/*! The message header for the \em one-round protocol. */
typedef struct {
  node_t* from;         /*!< */
  msg_rcpt_t* rcpt;     /*!< */
  msg_type_t type;      /*!< */
} msg_header_t;


/*! The message data for the \em one-round protocol. */
typedef struct {
  xmlDocPtr info;  /*!< The information sent over a message. */
} msg_data_t;


/*! The message used in the \em one-round protocol. */
typedef struct {
#if USE_MSG_ID
  msg_id_t     id;        /*!< The message id. */
#endif
  msg_header_t *header;   /*!< The header of the message. */
  msg_data_t   *data;     /*!< The data contained in this message. */
} msg_t;


/*! The definition of the function that should be called when a certain message arrives. */
typedef void (*msg_receive_func_t)(const msg_t* const);


msg_t* msg_new(node_t* const, msg_rcpt_t* const, const msg_type_t, const xmlDocPtr);
void msg_destroy(msg_t*);
void msg_destroy_full(msg_t*);
gboolean msg_enabled(void);

void msg_init(node_t* const);
void msg_finalize(void);
gboolean msg_enaled(void);
void msg_print(const msg_t* const);

void msg_send(msg_t* const);
void msg_receive(msg_t* const);

void msg_add_handle(const msg_type_t, const msg_receive_func_t);
void msg_remove_handle(const msg_type_t);

msg_t* msg_get_current_message(void);

xmlDocPtr msg_to_xml(const msg_t* const);
msg_t* msg_from_xml(const xmlDocPtr);


#endif /* msg.h */
