/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _NODE_H
#define _NODE_H

#include <glib.h>
#include <libxml/tree.h>


/*! The maximum length of the hostname of a node. */
#define NODE_NAME_MAX_LENGTH 32


/*! A node identifier. */
typedef guint node_id_t;

/*! The printf format to handle nodes identifiers. */
#define NODE_ID_FORMAT "%" G_GUINT32_FORMAT


/*! A data structure that represents a node. */
typedef struct {
  node_id_t id;      /*! The node identifier. */
  gchar* hostname;   /*! The node hostname. */
  guint port;        /*! The node port number. */
} node_t;

/*! */
#define NODE_PORT_FORMAT "%u"


node_t* node_new(const node_id_t, gchar* const, const guint);
void node_destroy(node_t* const);

gint node_compare(const node_t* const, const node_t* const);
gint node_compare_full(const node_t* const, const node_t* const, gpointer);
gint node_compare_by_id(const node_t* const, const node_t* const); 

gint node_id_compare(const node_id_t, const node_id_t);

xmlDocPtr node_to_xml(const node_t* const);
node_t* node_from_xml(const xmlDocPtr);


#endif /* node.h */
