/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <signal.h>

//#include <glib.h>

#include "configuration.h"
#include "gmsd.h"
#include "log.h"
#include "signals.h"


/*! The configuration file name */
extern char* config_filename;

static void signals_handle(int signal);

/*! Internal variable used to identify if this module has already been initialized */
static gboolean enabled = FALSE;


/** 
 * Initializes everything related to signal handling.
 */
void signals_init() 
{
  log_debug("[signals init] init");

  signal(SIGINT,  signals_handle);
  signal(SIGHUP,  signals_handle);

  enabled = TRUE;
  log_debug("[signals init] exit");
}


/** 
 * Destroys everything related to signal handling.
 */
void signals_finalize()
{
  log_debug("[signals finalize] init");

  signal(SIGINT,  SIGNAL_DEFAULT_BEHAVIOR);
  signal(SIGHUP,  SIGNAL_DEFAULT_BEHAVIOR);

  enabled = FALSE;
  log_debug("[signals finalize] exit");
}


/**
 * Informs if this modules has been initialized.
 * \return A boolean that indicates if this modules has been initialized.
 */
gboolean signals_enabled()
{
  return enabled;
}


/**
 * Function responsible for handling signals.
 * \param signal The signal received.
 */
static void signals_handle(int signal) 
{
  switch (signal) {
    
  case SIGINT:
    log_warning("SIGINT received. Exiting.");
    gmsd_stop();
    break;

  case SIGHUP: 
    config_read();
    log_info("Configuration reloaded from file '%s'", config_filename);
    break;

  default:
    break;
  }
}
