/* GMSD - Group Membership Service Daemon
 * Copyright (C) 2003 Nelio Pereira (nelio@ime.usp.br)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _TIMEOUT_H
#define _TIMEOUT_H

#include <glib.h>

/*! The definition of function called by the timer. */
typedef void (*timer_func_t)(gpointer data);

/*! The timer type. */
typedef struct {
  guint id;              /*!< The source id related to this timer. */
  guint timeout;         /*!< The current timeout associated to this timer. */
  timer_func_t func;     /*!< The function that this timer executes. */
  gpointer func_data;    /*!< The data to be passed to the above function. */
  gboolean executing;    /*!< whether this timer is executing. */
} mytimer_t;

/*! The default value of the timer_id. */
#define TIMER_ID_DEFAULT -1


mytimer_t* timer_new(const timer_func_t, const gpointer);
void timer_destroy(mytimer_t* const);

void timer_set(mytimer_t* const, const guint);
void timer_cancel(mytimer_t* const);
gboolean timer_enabled(const mytimer_t* const);

#endif /* timeout.h */
