#!/usr/bin/perl


#
# Nelio Pereira - nelio@ime.usp.br
# 2003-08-04
#
# create_config.pl
# Creates configuration files for a list of nodes.
#
#

use Getopt::Std;

require 'misc/libutil.pl';

$CONF_FILENAME = "gmsd.conf";


sub usage
{
    print "Usage: " . $0 . " [-h] [-n <neighbor timeout>]\n";
    print "                  [-p <probe timeout>] [-d <messages delay>]\n";
    print "                  [-l] -f <nodes file name>\n";
    print "       -l: loopback multicast messages.\n";
    print "       -f: nodes filename locate at config/\n";
    print "       -h: this help message\n";
    print "\n";
    exit;
}



sub generate_config
{
    my ($suffix, $comm_port) = @_;
    my ($conf);
    
    my ($user, $group);
    $user  = (getpwuid($<))[0];
    $group = (getgrgid($())[0];

    $conf = qq~#
# GROUP MEMBERSHIP SERVICE DAEMON - configuration file
#
	
# Neighbor check timeout
timeout.neighbor = $NEIGHBOR_TIMEOUT
    
# Leader check timeout
timeout.probe = $PROBE_TIMEOUT
    
# Delay timeout
timeout.delay = $MESSAGE_DELAY

# Which port to listen for messages
comm.port = $comm_port

# Which multicast address to listen for messages
comm.mcast.address = "224.2.127.254"

# Which multicast port to listen for messages
comm.mcast.port = 4444

# Whether should loopback multicast messages
comm.mcast.loop = $LOOPBACK

# The time-to-live used in multicast communication
comm.mcast.ttl = 1

# the log file name
log.filename = "$ENV{PWD}/${LOG_FILE}${suffix}"
    
# the log level
log.level = "info"

# the group identifier persistence file
gid.filename = "$ENV{PWD}/${GID_FILE}${suffix}"
    
# the file where the membership will be saved
ipc.filename = "$ENV{PWD}/${MEMBERSHIP_FILE}${suffix}"

# Possible members of the group.
# This file should contain a node per line.
node.list.file = "$ENV{PWD}/config/${NODES_FILENAME}"

# The user to run the daemon
run_as.user = "$user"

# The group to run the daemon
run_as.group = "$group"
~;
    
    return $conf;
}


####################
# MAIN
###################################

getopts('n:p:d:f:lh');

if ($opt_h || ! $opt_f) {
    usage;
}

$NODES_FILENAME = $opt_f;
$FILE = "config/${NODES_FILENAME}";

$num_lines = `wc -l $FILE`;

if (! -R "config/${NODES_FILENAME}") {
    print "Error: either file config/${NODES_FILENAME} does not exist or it is not readable.\n";
    exit;
}

$LOOPBACK = $opt_l ? "true" : "false";

open ARQ, $FILE or die $!;

$MESSAGE_DELAY = $opt_d || 400;
$NEIGHBOR_TIMEOUT = $opt_n || $MESSAGE_DELAY * $num_lines;
$PROBE_TIMEOUT = $opt_p || 300000;

$id = 1;
foreach $line (<ARQ>) {
    $line =~ /^(.+)\s+(.+)$/;
    $suffix = make_suffix($1, $id);
    $conf = generate_config($suffix, $2);
    open CONF, ">config/${CONF_FILENAME}".$suffix or die $!;
    print CONF $conf;
    close CONF;
    
    $id++;
}

close ARQ;
