#!/usr/bin/perl

#
# Nelio Pereira - nelio@ime.usp.br
# 2003-08-04
#
# create_init_scripts.pl
# Creates init scripts to run each configured node gmsd service.
#
#

use Getopt::Std;

require 'misc/libutil.pl';


sub usage
{
    print "Usage: " . $0 . " [-h] -f <nodes file name>\n";
    print "\n";
    exit;
}



$SCRIPT = '#!/bin/bash

PATH=$PATH:/sbin:/usr/sbin

SUDO=/usr/bin/sudo
NICE=`which nice`" -n -15"
PIDOF=`which pidof`

GMSD=PWD/init/../../protocol/src/gmsd
GMSD_CONF=PWD/config/gmsd.confSUFFIX

export LANG=en_US

# Source function library.
. PWD/misc/functions

## start & stop functions

function start-gmsd {
    echo -n "Starting gmsd server: "
    if [ -e "PWD/pid/gmsd.pidSUFFIX" ]; then
	pid=`cat PWD/pid/gmsd.pidSUFFIX`
	out=`ps -p $pid | sed 1d`
	if [ "x$out" != "x" ]; then
	    echo_passed
	    echo
	    return $pidv
	fi
    fi
    
    $SUDO $NICE $GMSD -c $GMSD_CONF -d
    pid=`$PIDOF -s gmsd`
    if [ "x$pid" = "x" ]; then
	echo_failure
    else
	echo_success
    fi
    echo $pid > PWD/pid/gmsd.pidSUFFIX

    echo
    return $pid
}

function stop-gmsd {
    echo -n "Stopping gmsd server: "
    if [ -e "PWD/pid/gmsd.pidSUFFIX" ]; then
	pid=`cat PWD/pid/gmsd.pidSUFFIX`
        out=`ps -p $pid | sed 1d`
	if [ "x$out" != "x" ]; then
	    kill -INT `cat PWD/pid/gmsd.pidSUFFIX`
	    echo_success
	else
	    echo_failure
	    echo
	    echo "gmsd server is not running" 
	fi
    else
	echo_failure
    fi
    echo
}

function status-gmsd {
    echo -n "Status of gmsd server: "
    if [ -e "PWD/pid/gmsd.pidSUFFIX" ]; then
	pid=`cat PWD/pid/gmsd.pidSUFFIX`
	out=`ps -p $pid | sed 1d`
	if [ "x$out" != "x" ]; then
	    echo_success
	    echo
	    return 1
	else
	    echo_failure
	    echo
	    return 0
	fi
    else
	echo_failure
	echo
	return 0
    fi
}


#
###############################

# See how we were called.
case "$1" in
    start)
	start-gmsd
	;;

    stop)
        stop-gmsd
	;;

    restart)
        $0 stop
        $0 start
        ;;

    status)
	status-gmsd
        ;;

    *)
        echo "Usage: gmsd {start|stop|status|restart}"
        exit 1
esac

exit 0
';


####################
# MAIN
###################################

getopts('f:h');

if ($opt_h || ! $opt_f) {
    usage;
}

$NODES_FILENAME = $opt_f;


if (! -R "config/${NODES_FILENAME}") {
    print "Error: either file config/${NODES_FILENAME} does not exist or it is not readable.\n";
    exit;
}

open ARQ, "config/${NODES_FILENAME}" or die $!;

$id = 1;
foreach $line (<ARQ>) {
    $line =~ /^([^\s]+)/;
    $suffix = make_suffix($1, $id);
    $id++;
    
    $file = "init/gmsd".$suffix;

    open FILE, ">".$file or die "Cannot open $file: $!";

#    print FILE qq~#!/bin/bash
#syncdate
#/bin/nice -n -15 $ENV{PWD}/init/../../protocol/src/gmsd -c $ENV{PWD}/config/gmsd.conf${suffix} \$*
#~;

#    print FILE qq~#!/bin/bash
#/usr/bin/sudo /usr/bin/nice -n -15 $ENV{HOME}/bin/gmsd -c $ENV{PWD}/config/gmsd.conf${suffix} \$*
#~;

#    print FILE qq~#!/bin/bash
#/usr/bin/sudo nice -n -15 $ENV{PWD}/init/../../protocol/src/gmsd -c $ENV{PWD}/config/gmsd.conf${suffix} \$*
#pid=`/sbin/pidof gmsd`
#echo \$pid > $ENV{PWD}/pid/gmsd.pid${suffix}
#~;

    $init_script = $SCRIPT;
    $init_script =~ s/PWD/$ENV{PWD}/g;
    $init_script =~ s/SUFFIX/$suffix/g;

    print FILE $init_script;

    close FILE;
    chmod 0755, $file;
}

close ARQ;

