
# a raiz da instalacao da implementacao
_IMPL_ROOT=$HOME/membership_services

# a raiz do teste que estou realizando
_TEST_ROOT=$IMPL_ROOT/tests

# numero total de nos
_N_NODES=100

# numero total de repeticoes do experimento
_N_REP=2

# opcoes para o gera_plot
_GERA_PLOT_OPT="-t 50000 -v"

export _IMPL_ROOT _TEST_ROOT _N_NODES _N_REP _GERA_PLOT_OPT


#####
#
# Incializa a sequencia de ns passada como argumento.
#
###
function run()
{
    if [ "x$1" == "x" ]; then
	echo "run(): missing the list argument!"
	return
    fi

    list=$1

    pushd $_IMPL_ROOT/run_env
    make clean-env
    make
    
    count=1
    for j in $list; do 
	i=`printf "%.3d" $j`
	
	echo "Inicializando processador $i ..."
	./init/gmsd-${HOSTNAME}-$i start 
	sleep 2
	
	aux=`grep -h group-id membership/*`
	x=`echo -ne "$aux" | wc -l`
	x=$((x+1))
	y=`echo -ne "$aux" | sort | uniq | wc -l`
	while [[ "$x" -ne "$count" ]] || [[ "$y" -ne 1 ]]; do
	    echo "["`date`"] x '$x' y '$y' count '$count'" >> /tmp/run.$$
	    sleep 2
	    aux=`grep -h group-id membership/* | sort`
	    x=`echo -ne "$aux" | wc -l`
	    x=$((x+1))
	    y=`echo -ne "$aux" | sort | uniq | wc -l`
	done

	count=$((count+1))
    done 
    
}


#####
#
# Roda um experimento para a inicializacao da sequencia passada como argumento.
#
###
function run-inicializacao()
{
    run "$1"

    # agora mata todos
    kill -INT `cat pid/* | xargs`
#    killall gmsd
    sleep 2

    # coleta os dados
    popd
    cp -a $_IMPL_ROOT/run_env/log .
    sort log/* > log/logao-total
    
    # agora tira as linhas com as finalizacoes
    ../../bin/limpa_finalizacao.pl < log/logao-total > log/logao
}


#####
#
# Roda um experimento para a finalizacao da sequencia de nos passada como argumento.
#
###
function run-finalizacao()
{
    if [ "x$1" == "x" ]; then
	echo "run-finalizacao(): missing the list argument!"
	return
    fi

    list=`seq 1 $_N_NODES`
    run "$list"

    list=$1
     
    count=$_N_NODES
    for j in $list; do 
	count=$((count-1))

	i=`printf "%.3d" $j`
	
	old=`grep -h group-id membership/* | sort | uniq`

	echo "Finalizando processador $i ..."
	./init/gmsd-${HOSTNAME}-$i stop 

# espera alguem notar a mudanca de grupo ...
	aux=`grep -h group-id membership/* | sort | uniq`
	while [ "$aux" = "$old" ]; do
	    sleep 2
	    aux=`grep -h group-id membership/* | sort | uniq`
	done

# ... agora espera que todos percebam ...
        aux=`grep -h 30$i membership/* | wc -l`
        while [ "$aux" -gt 0 ]; do
            sleep 2
            aux=`grep -h 30$i membership/* | wc -l`
        done

# ... e agora espera que todos tenham se unido ao mesmo grupo.
	aux=`grep -h group-id membership/* | sort | uniq | wc -l`
	while [[ "$aux" -ne 1 ]] && [[ "$count" -gt 0 ]]; do
	    sleep 1
	    aux=`grep -h group-id membership/* | sort | uniq | wc -l`
	done
	
    done 
    
    # agora mata todos
#    kill -KILL `cat pid/* | xargs`
#    killall gmsd
#    sleep 2

    # coleta os dados
    popd
    cp -a $_IMPL_ROOT/run_env/log .
    sort log/* > log/logao-total
   
    # agora tira as linhas ate o formacao do grupo
    ../../bin/limpa_inicializacao.pl < log/logao-total > log/logao
}


#####
#
# Roda a finalizacao multipla dos ns
#
##
function run-finalizacao-multipla()
{
    if [ "x$1" == "x" ]; then
	echo "run-finalizacao-multipla(): missing the node argument!"
	return
    fi
    
    list=`seq 1 $_N_NODES`
    run "$list"
    
    node=$1
    count=`printf "%.3d" $node`

    rm -rf /tmp/yy.$$
    mkdir /tmp/yy.$$
    cp -a $_IMPL_ROOT/run_env/pid/* /tmp/yy.$$
    rm /tmp/yy.$$/*$count
    
    old=`grep -h group-id membership/* | sort | uniq`

    echo -ne "Finalizando todos os ns, exceto o $node ...  "
    kill -INT `cat /tmp/yy.$$/* | xargs`
    echo "done!"
    echo "Aguardando a reestruturao do grupo... "

    x=$((30000+$node))
#    aux=`grep -h 300.. membership/* | grep -v $x`
    aux=`grep -h "<port>30..." membership/* | grep -v $x`
    while [ "x$aux" != "x" ]; do
	sleep 2
#	aux=`grep -h 3000. membership/* | grep -v $x`
	aux=`grep -h "<port>30..." membership/* | grep -v $x`
    done

    # agora mata o que sobrou
#    killall gmsd
    kill -INT `cat pid/*$count | xargs`


    # coleta os dados
    popd
    cp -a $_IMPL_ROOT/run_env/log .
    sort log/* > log/logao-total
   
    # agora tira as linhas ate o formacao do grupo
    ../../bin/limpa_inicializacao.pl < log/logao-total > log/logao
}




function calc_col-member()
{
   echo "6*$1-4" | bc
   return
}


function calc_col-gid()
{
    echo "6*$1-2" | bc
    return
}


function calc_col-gid-count()
{
    echo "6*$1" | bc
    return
}


#####
#
# Gera a configuracao do gnuplot
#
###
function gera_gnuplot()
{
    in_filename=$1
    out_filename=$2
    title=$3
    xlabel=$4
    ylabel=$5
    nome_curva=$6
    funcao=$7
    xr=$(($_N_NODES+3))
    yr=$(($_N_NODES*35))
    conf=`cat <<EOF
set title "Tempo de Estabilizao dos Ns - $title"
set xlabel "${xlabel}"
set ylabel "${ylabel}"
set xtics 5
set grid
set xr [0:$xr]
set yr [0:$yr]
set terminal epslatex color
set output "${out_filename}.eps"
set size 1.25, 1.25
plot 
EOF`

    n=$(($_N_NODES-1))
    for i in `seq 1 $n`; do
	x=`$funcao $i`
	conf=`echo -ne "$conf\"${in_filename}\" using 1:$x title '${nome_curva} $i' with linespoints , "`
    done
    x=`$funcao $_N_NODES`
    conf=`echo -ne "$conf\"${in_filename}\" using 1:$x title '${nome_curva} $_N_NODES' with linespoints  \n"`
    
    conf=`echo "$conf"; cat <<EOF
set terminal pbm color
set size 2,2
set output "${out_filename}.pbm"
replot
EOF`
    
    echo "$conf" 
    return 
}



function gera_gnuplot-member()
{
    gera_gnuplot "$1" "$2" "Inicializao - Viso do Grupo (member)" "$3" "Tempo (ms)" "$4" "calc_col-member"
}


function gera_gnuplot-gid()
{
    gera_gnuplot "$1" "$2" "Inicializao - Viso do Grupo (gid)" "$3" "Tempo (ms)" "$4" "calc_col-gid"
}


function gera_gnuplot-gid-count()
{
    gera_gnuplot "$1" "$2" "Inicializao - Mudanas de GID" "$3" "Quantidade" "$4" "calc_col-gid-count"
}




#####
#
# Gera os graficos para um certo conjunto de dados.
#
###
function gera_graph()
{
    dir=$1

    # gera a configuracao  do gnuplot
    gera_gnuplot-member "plot-groups.dat" "plot-groups-member" "Grupos" "N" > $dir/plot-groups-member.p
    gera_gnuplot-gid "plot-groups.dat" "plot-groups-gid" "Grupos" "N" > $dir/plot-groups-gid.p
    gera_gnuplot-gid-count "plot-groups.dat" "plot-groups-gid-count" "Grupos" "N" > $dir/plot-groups-gid-count.p
    
    gera_gnuplot-member "plot-nodes.dat" "plot-nodes-member" "Ns" "Grupo" > $dir/plot-nodes-member.p
    gera_gnuplot-gid "plot-nodes.dat" "plot-nodes-gid" "Ns" "Grupo" > $dir/plot-nodes-gid.p
    
    # gera os graficos
    cd $dir
    gnuplot plot-groups-member.p
    gnuplot plot-groups-gid.p
    gnuplot plot-nodes-member.p
    gnuplot plot-nodes-gid.p
    cd ..
}


