#!/usr/bin/perl

use Getopt::Std;

require '../bin/libparse.pl';

$_DEBUG = 0;
$_COMM  = 0;
$_VERBOSE = 0;
$_GROUPS = 1;
$_CRESC = 1;
$_BROADCAST_TIME = 0;

sub usage
{
    print "Usage: $0 [-g] [-c] [-n] [-d] [-v] [-h] -f <log file to be analyzed> [-t <broadcast time>]\n";
    print "       -f: file to be parsed\n";
    print "       -n: produces per node information. Default: disabled (per group information)\n";
    print "       -d: indicates that we had a top-down finalization. Default: disabled (bottom-up initialization)\n";
    print "       -g: debug information\n";
    print "       -c: consider communication delays\n";
    print "       -t: change each broadcast time to this time (microseconds)\n";
    print "       -v: verbose\n";
    print "       -h: this message\n";
    print "\n";
    exit;
}


sub nodesup_str
{
    my ($nodesup) = @_;
    return join(',', sort {$a <=> $b} keys %$nodesup);
}


sub print_line
{
    local ($aux);

    print $grupo;
    
    $aux = 1;
#    foreach $node (sort {$a <=> $b} keys %{$grupos{$grupo}}) {
    foreach $node (sort {$a <=> $b} keys %nodesup) {
	while ($aux < $node) {
	    print "\t-\t-\t-";
	    $aux++;
	}
	
	print '<'.$node.'>' if $_DEBUG;

	$m = $grupos{$grupo}->{"$node"}->{'M'} / 1000;
	$g = $grupos{$grupo}->{"$node"}->{'G'} / 1000;
	
	print "\t" . $m . "\t" . $g . "\t" . $grupos{$grupo}->{"$node"}->{'gid-count'};
#	printf "\t%.3f\t%.3f", $m, $g;
	$aux++;
    }
    
    while ($aux <= $ENV{'_N_NODES'}) {
	print "\t-\t-\t-";
	$aux++;
    }

    print "\n";
}



##################
# MAIN
###########

getopts('gdcvf:nht:');

if ($opt_g) {
    $_DEBUG = 1;
}

if ($opt_c) {
    $_COMM = 1;
}

if ($opt_v) {
    $_VERBOSE = 1;
}

if (defined $opt_f) {
    $filename = $opt_f;
}

if ($opt_n) {
    $_GROUPS = 0;
}

if ($opt_d) {
    $_CRESC = 0;
}

if ($opt_t) {
    $_BROADCAST_TIME = $opt_t;
}

if ((! $filename) || ($opt_h)) {
    &usage;
}


#
#
# Begin processing
#
#

mylog("Parsing the log... ") if $_VERBOSE;

$events = parse_log($filename);

mylog("done!") if $_VERBOSE;
mylog("Consolidating events... ") if $_VERBOSE;

$table = consolidate_events($events);

mylog("done!") if $_VERBOSE;
mylog("Starting generating info... ") if $_VERBOSE;

$grupo = 0;
foreach $node (1..$ENV{'_N_NODES'}) {
    $nodesup{"$node"} = 1;
}

%grupos = ();
$init_time = 0;

$last_down = 0;

$records = $table->{'records'};


#
#
# First we adjust the time to eliminate communication delays
#
#

$sortv = $records;

if (! $_COMM) {

    mylog("Eliminating communication delays... ") if $_VERBOSE;

#    @aux = sort sort_records @$records;
#    $sortv = \@aux;

    %delta = ();
    foreach $event (@$sortv) {
	
	$timestamp = $event->{'timestamp'};
	$node = $event->{'hostname'};

	$message = $table->{'messages'}->{$event->{'msg_id'}};
	if (($event->{'type'} eq 'MESSAGE_RECEIVED') && (defined $message)) {
	    if ($message->{'msg_from'} == $node) {
#		print STDERR "!! $node - $message->{'msg_id'} !!\n";
		next;
	    }
	    
	    $delta{$node} = ($timestamp - $message->{'timestamp'} - 1);
	    $delta{$node} -= $_BROADCAST_TIME if $_BROADCAST_TIME;
#	    print STDERR "$node $event->{msg_id} $delta{$node}\n";
	} 
	
	$event->{'timestamp'} -= $delta{$node};
    }

    mylog("done!") if $_VERBOSE;
}


#
#
# Then we process the log to generate stabilization time
#
#

mylog("Generating stabilization time... ") if $_VERBOSE;

foreach $event (sort sort_records @$sortv) 
{
    $timestamp = $event->{'timestamp'};
    $node = $event->{'hostname'};

#    print STDERR "$timestamp - $node : $event->{'type'}\n";

    if ($event->{'type'} eq 'DOWN') 
    {
	if (exists $grupos{$grupo}) {
	    &print_line if $_GROUPS;
	}

	delete $nodesup{"$node"};

	if (! $last_down) {
	    $init_time = $timestamp;
	    $grupo++;
	}
	$last_down = 1;
    } elsif ($event->{'type'} eq 'JOIN') {
	$last_down = 0;
        $members = $event->{'nodes'}->{"$node"}->{'members'};
        $time = $timestamp - $init_time;

        $ns = nodesup_str \%nodesup;
        if ($ns ne $members) {
#	    print STDERR "=> $node - " . $event->{'nodes'}->{$node}->{'gid'} . " - '$ns' <> '$members'\n";
            next;
        }

	if (! exists $grupos{$grupo}->{"$node"}->{'M'}) {
	    $grupos{$grupo}->{"$node"}->{'M'} = $time;
	}

        $grupos{$grupo}->{"$node"}->{'gid-count'}++;
	$grupos{$grupo}->{"$node"}->{'G'} = $time;
    }

}


if ($_GROUPS) {

# falta imprimir os dados do ultimo grupo
    &print_line;

} else {
    
    foreach $node (1..$ENV{'_N_NODES'})
    {
	print $node;
	
	foreach $grp (1..$grupo) 
	{
	    if ($grupos{$grp}->{"$node"}->{'M'}) 
	    {
		print '<'.$node.'>' if $_DEBUG;
		
		$m = $grupos{$grp}->{"$node"}->{'M'} / 1000;
		$g = $grupos{$grp}->{"$node"}->{'G'} / 1000;
		
		print "\t" . $m . "\t" . $g . "\t" . $grupos{$grp}->{"$node"}->{'gid-count'};
	    } else {
		print "\t-\t-\t-";
	    }
	    
	}
	
	print "\n";
    }
}


mylog("done!") if $_VERBOSE;
